import argparse
import sys
import rasterio.enums
import logging
from shapely.wkt import loads


def main():
    resampling_methods = [method.name for method in rasterio.enums.Resampling]

    parser = argparse.ArgumentParser(
        description="Generate co-locations between two products."
    )

    parser.add_argument("--product1-id", type=str, help="Path of the first product.")
    parser.add_argument("--product2-id", type=str, help="Path of the second product")
    parser.add_argument(
        "--destination-folder",
        default="/tmp",
        nargs="?",
        type=str,
        help="Folder path for the output.",
    )
    parser.add_argument(
        "--delta-time",
        default=30,
        nargs="?",
        type=int,
        help="Maximum time in minutes between two product acquisitions.",
    )
    parser.add_argument(
        "--minimal-area",
        default="1600km2",
        nargs="?",
        type=str,
        help="Minimal intersection area in square kilometers.",
    )
    parser.add_argument(
        "--listing",
        default=True,
        action="store_true",
        help="Create a listing of co-located files.",
    )
    parser.add_argument(
        "--no-listing",
        dest="listing",
        action="store_false",
        help="Do not create a listing.",
    )
    parser.add_argument(
        "--product-generation",
        default=True,
        action="store_true",
        help="Generate a co-location product.",
    )
    parser.add_argument(
        "--no-product-generation",
        dest="product_generation",
        action="store_false",
        help="Do not generate a co-location product.",
    )
    parser.add_argument(
        "--listing-filename",
        nargs="?",
        type=str,
        help="Name of the listing file to be created.",
    )
    parser.add_argument(
        "--colocation-filename",
        nargs="?",
        type=str,
        help="Name of the co-location product to be created.",
    )
    parser.add_argument(
        "--resampling-method", type=str, default="nearest", choices=resampling_methods
    )
    parser.add_argument(
        "--footprint1",
        type=str,
        default=None,
        help="Optional argument to provide a WKT footprint for product1.",
    )
    parser.add_argument(
        "--footprint2",
        type=str,
        default=None,
        help="Optional argument to provide a WKT footprint for product2.",
    )
    parser.add_argument(
        "--config",
        type=str,
        help="Configuration file to use instead of the " "default one.",
    )
    parser.add_argument("--debug", action="store_true", default=False)
    parser.add_argument("-v", "--version", action="store_true", help="Print version")

    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(
            level=logging.DEBUG,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
            format="%(asctime)s [%(levelname)s]: %(message)s",  # Define the log message format
            datefmt="%Y-%m-%d %H:%M:%S",  # Define the date/time format
        )
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.DEBUG)
        coloc_logger = logging.getLogger("coloc_sat")
        coloc_logger.setLevel(logging.DEBUG)
    else:
        logging.basicConfig(
            level=logging.INFO,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
            format="%(asctime)s [%(levelname)s]: %(message)s",  # Define the log message format
            datefmt="%Y-%m-%d %H:%M:%S",  # Define the date/time format
        )
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.INFO)
        coloc_logger = logging.getLogger("coloc_sat")
        coloc_logger.setLevel(logging.INFO)

    from coloc_sat.intersection import __version__

    if args.version:
        print(__version__)
        sys.exit(0)
    import coloc_sat
    from coloc_sat.generate_coloc import GenerateColoc

    logger.info(f"The script is executed from {__file__}")

    # Check for missing required arguments
    if not args.product1_id or not args.product2_id:
        parser.error("product1-id and product2-id are required arguments.")

    # Information for the user about listing / co-location product creation
    if args.listing is True:
        logger.info(
            "A listing of the co-located products will be created. To disable, use --no-listing."
        )
    if args.product_generation is True:
        logger.info(
            "Co-location products will be created. To disable, use --no-product-generation."
        )

    logger.warning(
        "WARNING : product colocation has only been tested on _ll_gd SAR products."
    )

    if args.footprint1 is not None:
        args.footprint1 = loads(args.footprint1)

    if args.footprint2 is not None:
        args.footprint2 = loads(args.footprint2)
        print(args.footprint2)

    generator = GenerateColoc(**vars(args))
    status = generator.save_results()

    logger.info("Coloc python program successfully ended")
    sys.exit(status)



import argparse
import sys
import rasterio.enums
import logging

def main():
    resampling_methods = [method.name for method in rasterio.enums.Resampling]

    parser = argparse.ArgumentParser(
        description="Generate co-locations using .parquet file containing intersections. The kind of .parquet file used by this script is generated by Jean-François Piolle from Ifremer."
    )

    parser.add_argument(
        "--parquet",
        type=str,
        help="Parquet file containing intersections",
        required=True,
    )
    parser.add_argument(
        "--filter-dataset-unique",
        type=str,
        default=None,
        help="Can be 'ref' or 'match', to indicate which dataset must be filtered to remove duplicated files. By default, no dataset is filtered.",
    )
    parser.add_argument(
        "-d",
        "--destination-folder",
        default=None,
        nargs="?",
        type=str,
        help="Folder path for the output. Can also be given individually for each coloc in the .parquet file as 'destination_folder' column. Optional, but necessary either in arguments or in the .parquet.",
    )
    parser.add_argument(
        "--delta-time",
        default=30,
        nargs="?",
        type=int,
        help="Maximum time in minutes between two product acquisitions.",
    )
    parser.add_argument(
        "--minimal-area",
        default="1600km2",
        nargs="?",
        type=str,
        help="Minimal intersection area in square kilometers.",
    )
    parser.add_argument(
        "--product-generation",
        default=False,
        action="store_true",
        help="Generate a co-location product.",
    )
    parser.add_argument(
        "--resampling-method", type=str, default="nearest", choices=resampling_methods
    )
    parser.add_argument(
        "--config",
        type=str,
        help="Configuration file to use instead of the " "default one.",
    )
    parser.add_argument(
        "--parallel",
        action="store_true",
        help="Enable parallel processing on dask LocalCluster",
    )
    parser.add_argument(
        "--parallel-datarmor",
        action="store_true",
        help="Enable parallel processing on datarmor",
    )
    parser.add_argument(
        "--n-workers", type=int, help="Number of worker to use.", default=1
    )
    parser.add_argument(
        "--memory",
        type=int,
        help="Memory to use in GB (useful only in datarmor mode)",
        default=1,
    )

    parser.add_argument("--debug", action="store_true", default=False)
    parser.add_argument("-v", "--version", action="store_true", help="Print version")

    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(
            level=logging.DEBUG,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
            format="%(asctime)s [%(levelname)s]: %(message)s",  # Define the log message format
            datefmt="%Y-%m-%d %H:%M:%S",  # Define the date/time format
        )
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.DEBUG)
        coloc_logger = logging.getLogger("coloc_sat")
        coloc_logger.setLevel(logging.DEBUG)
    else:
        logging.basicConfig(
            level=logging.INFO,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
            format="%(asctime)s [%(levelname)s]: %(message)s",  # Define the log message format
            datefmt="%Y-%m-%d %H:%M:%S",  # Define the date/time format
        )
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.INFO)
        coloc_logger = logging.getLogger("coloc_sat")
        coloc_logger.setLevel(logging.INFO)

    from coloc_sat.version import __version__

    if args.version:
        print(__version__)
        sys.exit(0)

    from coloc_sat.tools import set_config
    set_config(args.config)

    from coloc_sat.parquet_coloc import coloc_from_parquet

    logger.info(f"The script is executed from {__file__}")

    if args.product_generation is True:
        logger.info("Co-location products will be created.")

    if args.filter_dataset_unique is not None and (
        args.filter_dataset_unique != "ref" and args.filter_dataset_unique != "match"
    ):
        raise ValueError(
            f"Invalid value {args.filter_dataset_unique} for --filter-dataset-unique . Must be 'ref' or 'match'"
        )

    coloc_from_parquet(**vars(args))

    logger.info("Coloc python program successfully ended")
    sys.exit(0)



import logging

import geopandas as gpd
from coloc_sat.generate_coloc import GenerateColoc
from coloc_sat.tools import (
    get_all_comparison_files,
    set_config,
    load_config,
    check_file_match_pattern_date,
)
from coloc_sat import init_cluster
from typing import Optional
import numpy as np
import os
from dask.distributed import Client
from dask import delayed, compute
import traceback
from datetime import timedelta

logger = logging.getLogger(__name__)


def setup_logger(filename):
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    # Create file handler which logs even debug messages
    fh = logging.FileHandler(filename)
    fh.setLevel(logging.INFO)

    # Create console handler with a higher log level
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)

    # Create formatter and add it to the handlers
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    fh.setFormatter(formatter)
    ch.setFormatter(formatter)

    # Add the handlers to the logger
    logger.addHandler(fh)
    logger.addHandler(ch)

    return logger, fh, ch


def teardown_logger(logger, fh, ch):
    # Remove handlers to stop logging to the file
    logger.removeHandler(fh)
    logger.removeHandler(ch)

    fh.close()
    ch.close()


def process_parquet_coloc(
    row,
    ds1,
    ds2,
    data_base_1,
    data_base_2,
    time_accuracy_1,
    time_accuracy_2,
    match_filename_1,
    match_filename_2,
    match_time_delta_sec_1,
    match_time_delta_sec_2,
    destination_folder,
    product_generation,
    delta_time,
    minimal_area,
    resampling_method,
    config,
    exception_to_log=True,
    log_name="coloc_hy2.log",
    status_name="coloc_hy2.status",
):
    if exception_to_log:
        log_path = os.path.join(destination_folder, log_name)
        status_path = os.path.join(destination_folder, status_name)
        os.makedirs(destination_folder, exist_ok=True)
        logger, fh, ch = setup_logger(log_path)
        status = 1
    else:
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)

    try:
        if config is not None:
            set_config(config)

        footprint1 = row["ref_geometry"]
        footprint2 = row["match_geometry"]

        # if (
        #    row["ref_granule"]
        #    != "S1A_IW_OCN__2SDV_20220531T235908_20220531T235933_043462_05308F_8771.SAFE"
        # ):
        #    continue

        o_files = get_all_comparison_files(
            start_date=row["match_start"] - match_time_delta_sec_2,
            stop_date=row["match_end"] + match_time_delta_sec_2,
            ds_name=ds2,
            input_ds=None,
            level=2,
            accuracy=time_accuracy_2,
        )

        if len(o_files) == 0:
            logger.warning(f"No file found for {row['match_granule']}")
            return 2

        if match_filename_2:
            o_file = None
            for f in o_files:
                if check_file_match_pattern_date(f, data_base_2, row["match_start"]):
                    o_file = f
            if not o_file:
                logger.warning(f"File {row['match_granule']} not found.")
                return 2
        else:
            o_file = o_files[0]

        ref_files = get_all_comparison_files(
            start_date=row["ref_start"] - match_time_delta_sec_1,
            stop_date=row["ref_end"] + match_time_delta_sec_1,
            ds_name=ds1,
            input_ds=None,
            level=2,
            accuracy=time_accuracy_1,
        )
        if len(ref_files) == 0:
            logger.warning(
                f"No file found for {row['ref_granule']}, {row['ref_start'] - match_time_delta_sec_1}, {row['ref_end'] + match_time_delta_sec_1}"
            )
            return 2

        if match_filename_1:
            r_file = None
            for f in ref_files:
                if check_file_match_pattern_date(f, data_base_1, row["ref_start"]):
                    r_file = f
            if not r_file:
                logger.warning(f"File {row['ref_granule']} not found.")
                return 2
        else:
            r_file = ref_files[0]

        logger.info(f"Found file {ref_files[0]}")
        logger.info(f"Found file {o_file}")

        logger.info(f"Process {row['ref_granule']} and {row['match_granule']}")

        generator = GenerateColoc(
            product1_id=o_file,
            product2_id=r_file,
            footprint1=footprint2,
            footprint2=footprint1,
            destination_folder=destination_folder,
            product_generation=product_generation,
            delta_time=delta_time,
            minimal_area=minimal_area,
            resampling_method=resampling_method,
            config=config,
        )
        status = generator.save_results()

    except Exception as e:
        if exception_to_log:
            logger.error(traceback.format_exc())
            status = 1
        else:
            raise e
    finally:
        if exception_to_log:
            teardown_logger(logger, fh, ch)
            with open(status_path, "w") as status_f:
                status_f.write(str(status))


def coloc_from_parquet(
    parquet: str,
    destination_folder: Optional[str],
    product_generation: bool,
    delta_time: int,
    minimal_area: str,
    resampling_method: str,
    filter_dataset_unique: Optional[str] = None,
    config: Optional[str] = None,
    parallel: Optional[bool] = False,
    parallel_datarmor: Optional[bool] = False,
    memory: Optional[int] = 2,
    n_workers: Optional[int] = 5,
    **kwargs,
):
    """
    Takes a .parquet file registering colocation between two sensors, and generate colocation files
    for each.
    Expected .parquet format :
    Index(['ref_geometry', 'ref_start', 'ref_end', 'ref_feature_type',
            'ref_dataset_id', 'ref_level', 'ref_granule', 'match_geometry',
            'match_start', 'match_end', 'match_feature_type', 'match_dataset_id',
            'match_level', 'match_granule', 'match_intersection', 'match_slice',
            'match_slice_start', 'match_slice_end', 'match_slice_geometry',
            'match_slices'],
            dtype='object')

    Parameters
    ----------
    parquet: str Path to parquet file
    destination folder: str Output folder for coloc files. Optional
    production_generation: bool Indicates if files must be created
    delta_time: int Time in minutes. Maximum time difference between the two acquisition for the coloc to be valid.
    minimal_area: str Minimal area for the coloc to be valid. Examples: 300km2, 10m2...
    resampling_method: str Value from rasterio.enums.Resampling. Only used when colocating gridded data.
    filter_dataset_unique: str Can be "ref" or "match", specifies which dataset will be filtered to keep unique values (filtered on granule name)
    """

    config_path = config
    if config_path is not None:
        set_config(config_path)

    conf_data = load_config()

    if parallel_datarmor:
        init_cluster(n_workers=n_workers, memory=memory)
    elif parallel:
        from dask.distributed import Client

        # Initiate Dask Client
        client = Client(processes=True, n_workers=n_workers)
        logger.info(f"Dashboard link: {client.dashboard_link}")

    ds1 = conf_data["dataset_name_1"]
    ds2 = conf_data["dataset_name_2"]
    t_acc_1 = conf_data["match_time_accuracy_1"]
    t_acc_2 = conf_data["match_time_accuracy_2"]
    match_filename_1 = conf_data["match_filename_1"]
    match_filename_2 = conf_data["match_filename_2"]
    match_time_delta_sec_1 = timedelta(seconds=conf_data["match_time_delta_seconds_1"])
    match_time_delta_sec_2 = timedelta(seconds=conf_data["match_time_delta_seconds_2"])

    sar_ds = ["S1", "RS2", "RCM"]
    if ds1 in sar_ds:
        data_base_1 = os.path.basename(conf_data["paths"][ds1]["L2"][0])
    else:
        data_base_1 = os.path.basename(conf_data["paths"][ds1][0])

    if ds2 in sar_ds:
        data_base_2 = os.path.basename(conf_data["paths"][ds2]["L2"][0])
    else:
        data_base_2 = os.path.basename(conf_data["paths"][ds2][0])

    prq = gpd.read_parquet(parquet)

    if "destination_folder" not in prq.columns and destination_folder is not None:
        prq["destination_folder"] = destination_folder
    elif "destination_folder" not in prq.columns and destination_folder is None:
        raise ValueError(
            "destination_folder is neither given in parquet or parameters."
        )

    if filter_dataset_unique:
        prq["time_diff"] = (prq["ref_start"] - prq["match_start"]).abs()
        prq = prq.sort_values(by=["time_diff"])

        if filter_dataset_unique == "ref":
            prq = prq.drop_duplicates(subset=["ref_granule"])
        elif filter_dataset_unique == "match":
            prq = prq.drop_duplicates(subset=["match_granule"])
        else:
            raise ValueError(
                f"Unsupported value {filter_dataset_unique} for filter_dataset_unique. Must be 'ref' or 'match'."
            )

    if parallel or parallel_datarmor:
        tasks = [
            delayed(process_parquet_coloc)(
                row,
                ds1,
                ds2,
                data_base_1,
                data_base_2,
                t_acc_1,
                t_acc_2,
                match_filename_1,
                match_filename_2,
                match_time_delta_sec_1,
                match_time_delta_sec_2,
                row["destination_folder"],
                product_generation,
                delta_time,
                minimal_area,
                resampling_method,
                config,
            )
            for _, row in prq.iterrows()
        ]
        results = compute(*tasks)
    else:
        for _, row in prq.iterrows():
            status = process_parquet_coloc(
                row,
                ds1,
                ds2,
                data_base_1,
                data_base_2,
                t_acc_1,
                t_acc_2,
                match_filename_1,
                match_filename_2,
                match_time_delta_sec_1,
                match_time_delta_sec_2,
                row["destination_folder"],
                product_generation,
                delta_time,
                minimal_area,
                resampling_method,
                config,
            )
            # if status == 1:
            #    raise RuntimeError(f"Fail to process, status {status}")



import logging
import argparse
import sys
import rasterio.enums


def main():
    resampling_methods = [method.name for method in rasterio.enums.Resampling]

    parser = argparse.ArgumentParser(description="Generate co-locations between a specified product and a mission.")

    parser.add_argument("--product1-id", type=str, help="Path of the first product.")
    parser.add_argument("--destination-folder", default='/tmp', nargs='?', type=str, help="Folder path for the output.")
    parser.add_argument("--delta-time", default=30, nargs='?', type=int,
                        help="Maximum time in minutes between two product acquisitions.")
    parser.add_argument("--minimal-area", default='1600km2', nargs='?', type=str,
                        help="Minimal intersection area in square kilometers.")
    parser.add_argument("--mission-name", nargs='?', type=str,
                        choices=['S1', 'RS2', 'RCM', 'HY2', 'ERA5', 'WS', 'SMOS', 'SMAP'],
                        help="Name of the dataset to be compared.")
    parser.add_argument("--input-ds", type=str, nargs='?',
                        help="Subset of mission products to compare with. It is a txt file that contains these paths.")
    parser.add_argument("--level", nargs='?', type=int, choices=[1, 2],
                        help="Product level (SAR missions only).")
    parser.add_argument("--listing", default=True, action="store_true",
                        help="Create a listing of co-located files.")
    parser.add_argument("--no-listing", dest="listing", action="store_false", help="Do not create a listing.")
    parser.add_argument("--product-generation", default=True, action="store_true",
                        help="Generate a co-location product.")
    parser.add_argument("--no-product-generation", dest="product_generation", action="store_false",
                        help="Do not generate a co-location product.")
    parser.add_argument("--listing-filename", nargs='?', type=str,
                        help="Name of the listing file to be created.")
    parser.add_argument("--colocation-filename", nargs='?', type=str,
                        help="Name of the co-location product to be created.")
    parser.add_argument("--resampling-method", type=str, default="nearest",
                        choices=resampling_methods)
    parser.add_argument("--config", type=str, help="Configuration file to use instead of the "
                                                   "default one.")
    parser.add_argument("--debug", action="store_true", default=False)
    parser.add_argument("-v", "--version", action="store_true", help="Print version")

    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(
            level=logging.DEBUG,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
            format="%(asctime)s [%(levelname)s]: %(message)s",  # Define the log message format
            datefmt="%Y-%m-%d %H:%M:%S",  # Define the date/time format
        )
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.DEBUG)
        coloc_logger = logging.getLogger("coloc_sat")
        coloc_logger.setLevel(logging.DEBUG)
    else:
        logging.basicConfig(
            level=logging.INFO,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
            format="%(asctime)s [%(levelname)s]: %(message)s",  # Define the log message format
            datefmt="%Y-%m-%d %H:%M:%S",  # Define the date/time format
        )
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.INFO)
        coloc_logger = logging.getLogger("coloc_sat")
        coloc_logger.setLevel(logging.INFO)

    from coloc_sat.intersection import __version__
    if args.version:
        print(__version__)
        sys.exit(0)
    import coloc_sat
    from coloc_sat.generate_coloc import GenerateColoc
    logger.info(f"The script is executed from {__file__}")

    # Check for missing required arguments
    if not args.product1_id or not args.mission_name:
        parser.error("product1-id and mission-name are required aprguments.")

    # rename mission_name by ds_name in the args because it is the argument used in GenerateColoc
    args.ds_name = args.mission_name
    del args.mission_name

    # Information for the user about listing / co-location product creation
    if args.listing is True:
        logger.info("A listing of the co-located products will be created. To disable, use --no-listing.")
    if args.product_generation is True:
        logger.info("Co-location products will be created. To disable, use --no-product-generation.")

    logger.warning("WARNING : product colocation has only been tested on _ll_gd SAR products.")

    generator = GenerateColoc(**vars(args))
    status = generator.save_results()

    logger.info("Coloc python program successfully ended")
    sys.exit(status)



import logging

logger = logging.getLogger(__name__)

from .tools import correct_dataset
import os
import numpy as np
import xarray as xr


def extract_wind_speed(smos_dataset):
    return smos_dataset.where((np.isfinite(smos_dataset.wind_dir)), drop=True)


class GetHy2Meta:
    def __init__(self, product_path, product_generation=False, footprint=None):
        self.product_path = product_path
        self.product_name = os.path.basename(self.product_path)
        self.product_generation = product_generation
        self._time_name = "time"
        self._longitude_name = "lon"
        self._latitude_name = "lat"
        if footprint is not None:
            self._footprint = footprint
        self._dataset = GetHy2Meta._open_nc(product_path).load()
        self.dataset = correct_dataset(self._dataset, self.longitude_name)

    @staticmethod
    def _open_nc(product_path):
        logger.debug(f"Opening {product_path}")
        ds = xr.open_dataset(product_path, decode_cf=False)
        # Convert all integer variables to float
        for var in ds.data_vars:
            if np.issubdtype(ds[var].dtype, np.integer):
                ds[var] = ds[var].astype("float64")

        ds = xr.decode_cf(ds)
        ds["lon"].values = np.where(
            ds["lon"].values > 180, ds["lon"].values - 360, ds["lon"].values
        )

        return ds

    @property
    def footprint(self):
        if hasattr(self, "_footprint") and self._footprint is not None:
            return self._footprint
        else:
            raise ValueError(
                f"No footprint for GetHY2Meta (product {self.product_path})"
            )

    @property
    def start_date(self):
        """
        Start acquisition time

        Returns
        -------
        numpy.datetime64
            Start time
        """
        return min(np.unique(self.dataset[self.time_name]))

    @property
    def stop_date(self):
        """
        Stop acquisition time

        Returns
        -------
        numpy.datetime64
            Stop time
        """
        return max(np.unique(self.dataset[self.time_name]))

    @property
    def longitude_name(self):
        """
        Get the name of the longitude variable in the dataset

        Returns
        -------
        str
            longitude name
        """
        return self._longitude_name

    @property
    def latitude_name(self):
        """
        Get the name of the latitude variable in the dataset

        Returns
        -------
        str
            latitude name
        """
        return self._latitude_name

    @property
    def time_name(self):
        """
        Get the name of the time variable in the dataset

        Returns
        -------
        str
            time name
        """
        return self._time_name

    @property
    def mission_name(self):
        """
        Get the mission name (ex : RADARSAT-2, Radarsat Constellation, SENTINEL-1A..)

        Returns
        -------
        str
            Mission name
        """
        return "Haiyang-2"

    @property
    def acquisition_type(self):
        """
        Gives the acquisition type (swath, truncated_swath,daily_regular_grid, model_regular_grid)

        Returns
        -------
        str
            acquisition type

        """
        return "swath"

    @property
    def dataset(self):
        """
        Getter for the acquisition dataset

        Returns
        -------
        xarray.Dataset
            Acquisition dataset
        """
        return self._dataset

    @dataset.setter
    def dataset(self, value):
        """
        Setter of attribute `self.dataset`

        Parameters
        ----------
        value: xarray.Dataset
            new Dataset
        """
        self._dataset = value

    @property
    def orbit_segment_name(self):
        """
        Gives the name of the variable for orbit segmentation in dataset (Ascending / Descending). If value is None,
        so the orbit hasn't orbited segmentation

        Returns
        -------
        str | None
            Orbit segmentation variable name in the dataset. None if there isn't one.
        """
        return None

    @property
    def has_orbited_segmentation(self):
        """
        True if there is orbit segmentation in the dataset

        Returns
        -------
        bool
            Presence or not of an orbit segmentation
        """
        if self.orbit_segment_name is not None:
            return True
        else:
            return False

    @longitude_name.setter
    def longitude_name(self, value):
        self._longitude_name = value

    @latitude_name.setter
    def latitude_name(self, value):
        self._latitude_name = value

    @property
    def unecessary_vars_in_coloc_product(self):
        """
        Get unecessary variables in co-location product

        Returns
        -------
        list[str]
            Unecessary variables in co-location product
        """
        return []

    @property
    def necessary_attrs_in_coloc_product(self):
        """
        Get necessary dataset attributes in co-location product

        Returns
        -------
        list[str]
            Necessary dataset attributes in co-location product
        """
        return []

    def rename_attrs_in_coloc_product(self, attr):
        """
        Get the new name of an attribute in co-location products from an original attribute

        Parameters
        ----------
        attr: str
            Attribute from the satellite dataset that needs to be renames for the co-location product.

        Returns
        -------
        str
            New attribute's name from the satellite dataset.
        """
        # no attributes to rename
        mapper = {}
        if attr in mapper.keys():
            return mapper[attr]
        else:
            return attr



"""Top-level package for coloc_sat."""

__author__ = """Yann Reynaud"""
__email__ = "yann.reynaud.2@ifremer.fr"

from .version import __version__
import struct
import socket
import fcntl
import os


def get_ip_address(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(
        fcntl.ioctl(
            s.fileno(),
            0x8915,  # SIOCGIFADDR
            struct.pack("256s", str.encode(ifname[:15])),
        )[20:24]
    )


def init_cluster(n_workers: int, memory: int):
    from dask_jobqueue import PBSCluster
    from dask.distributed import Client

    # see https://jobqueue.dask.org/en/latest/generated/dask_jobqueue.PBSCluster.html
    memory = f"{memory}GB"
    nprocs = 1
    cluster = PBSCluster(
        cores=nprocs,
        memory=memory,
        project="coloc_sat",
        queue="sequentiel",
        processes=nprocs,
        resource_spec="select=1:ncpus=%d:mem=%s" % (nprocs, memory),
        local_directory=os.path.expandvars("$TMPDIR"),
        interface="ib1",  # workers interface (routable to queue ftp)
        walltime="00:20:00",
        scheduler_options={"interface": "ib0"},
        job_extra=["-m n"],
    )  # if scheduler is on queue 'ftp'

    cluster.scale(jobs=n_workers)  # ask for 200 jobs

    c = Client(cluster)
    # getting a working dashboard link is little tricky on datarmor
    print(
        "Client dashboard: %s"
        % c.dashboard_link.replace(get_ip_address("ib0"), get_ip_address("bond1"))
    )
    return c
